@@ -1,32 +1,15 @@
-// The MIT License (MIT)
-
-// Copyright (c) 2020 наб <nabijaczleweli@nabijaczleweli.xyz>
-
-// Permission is hereby granted, free of charge, to any person obtaining a copy of
-// this software and associated documentation files (the "Software"), to deal in
-// the Software without restriction, including without limitation the rights to
-// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
-// the Software, and to permit persons to whom the Software is furnished to do so,
-// subject to the following conditions:
-
-// The above copyright notice and this permission notice shall be included in all
-// copies or substantial portions of the Software.
-
-// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
-// FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
-// COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
-// IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
-// CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+// SPDX-License-Identifier: MIT
 
 
 #pragma once
 
 
+#include "sha1.hpp"
 #include "util.hpp"
 #include <array>
 #include <cstdint>
 #include <fmt/format.h>
+#include <libintl.h>
 #include <map>
 #include <optional>
 #include <string>
@@ -39,8 +22,7 @@ namespace klapki {
 
 
 namespace klapki::state {
-	using sha_t             = std::uint8_t[20];
-	using shaa_t            = std::array<std::uint8_t, 20>;
+	using shaa_t            = std::array<std::uint8_t, sizeof(sha_t)>;
 	using nonbase_dirname_t = std::optional<std::string>;  // Use SUB to copy from previous entry
 
 	struct boot_order_flat {
@@ -48,8 +30,7 @@ namespace klapki::state {
 		std::size_t order_cnt;
 	};
 	struct boot_order_structured {
-		/// [chunk, ours?]
-		std::vector<std::pair<std::vector<std::uint16_t>, bool>> order;
+		std::vector<std::uint16_t> foreign, ours;
 	};
 	using boot_order_t = std::variant<boot_order_flat, boot_order_structured>;
 
@@ -80,9 +61,9 @@ namespace klapki::state {
 		std::vector<std::string> variants;  // entries NUL-terminated; list empty-terminated; kept unique, order-preserving
 		std::vector<stated_config_entry> wanted_entries;
 
-		static void parse(stated_config & into, const void * data, std::size_t size);
+		static void parse(stated_config & into, bool may_decompress, const void * data, std::size_t size);
 
-		std::vector<std::uint8_t> serialise() const;
+		std::vector<std::uint8_t> serialise(bool may_compress) const;
 
 
 		bool operator==(const stated_config & other) const noexcept;
@@ -94,9 +75,9 @@ namespace klapki::state {
 		std::map<std::uint16_t, boot_entry> entries;
 		stated_config statecfg;
 
-		std::optional<std::string> commit(std::string_view us, const state & original_state) const;
+		std::optional<std::string> commit(const char * us, bool may_compress_statecfg, const state & original_state) const;
 
-		static std::variant<state, std::string> load(std::string_view us);
+		static std::variant<state, std::string> load(const std::string_view & us, bool may_decompress_statecfg);
 	};
 }
 
@@ -122,17 +103,15 @@ struct fmt::formatter<klapki::state::boot_order_t> {
 		std::visit(klapki::overload{
 		               [&](const klapki::state::boot_order_flat & bord) { out = write(out, bord.order.get(), bord.order_cnt); },
 		               [&](const klapki::state::boot_order_structured & bord) {
-			               bool first = true;
-			               for(auto && el : bord.order) {
-				               if(!first)
-					               *out++ = ',';
-				               else
-					               first = false;
-
-				               *out++ = el.second ? '{' : '[';
-				               out    = write(out, el.first.data(), el.first.size());
-				               *out++ = el.second ? '}' : ']';
-			               }
+			               *out++ = '[';
+			               out    = write(out, bord.foreign.data(), bord.foreign.size());
+			               *out++ = ']';
+			               *out++ = ' ';
+			               *out++ = '&';
+			               *out++ = ' ';
+			               *out++ = '{';
+			               out    = write(out, bord.ours.data(), bord.ours.size());
+			               *out++ = '}';
 		               },
 		           },
 		           bord);
@@ -149,10 +128,11 @@ struct fmt::formatter<klapki::state::stated_config_entry> {
 	auto format(const klapki::state::stated_config_entry & ent, FormatContext & ctx) {
 		auto out = ctx.out();
 
-		out = format_to(out, "{{ {:04X}, \"{}\", {}{}{}, \"{}\", [",                                                                                          //
-		                ent.bootnum_hint,                                                                                                                     //
-		                ent.version,                                                                                                                          //
-		                ent.variant.empty() ? "" : "\"", ent.variant.empty() ? "(default)" : std::string_view{ent.variant}, ent.variant.empty() ? "" : "\"",  //
+		out = format_to(out, "{{ {:04X}, \"{}\", {}{}{}, \"{}\", [",  //
+		                ent.bootnum_hint,                             //
+		                ent.version,                                  // default variant
+		                ent.variant.empty() ? "" : "\"", ent.variant.empty() ? gettext("(default)") : std::string_view{ent.variant},
+		                ent.variant.empty() ? "" : "\"",  //
 		                ent.kernel_dirname);
 
 		bool first = true;
